package com.oracle.webservices.internal.api.message;

import java.util.HashSet;
import java.util.AbstractMap;
import java.util.Set;
import com.sun.istack.internal.Nullable;
import java.util.Iterator;
import com.sun.istack.internal.NotNull;
import java.util.IdentityHashMap;
import java.util.Map;

public abstract class BaseDistributedPropertySet extends BasePropertySet implements DistributedPropertySet
{
    private final Map<Class<? extends PropertySet>, PropertySet> satellites;
    private final Map<String, Object> viewthis;
    
    public BaseDistributedPropertySet() {
        this.satellites = new IdentityHashMap();
        this.viewthis = super.createView();
    }
    
    @Override
    public void addSatellite(@NotNull PropertySet satellite) {
        this.addSatellite(satellite.getClass(), satellite);
    }
    
    @Override
    public void addSatellite(@NotNull Class<? extends PropertySet> keyClass, @NotNull PropertySet satellite) {
        this.satellites.put(keyClass, satellite);
    }
    
    @Override
    public void removeSatellite(PropertySet satellite) {
        this.satellites.remove(satellite.getClass());
    }
    
    public void copySatelliteInto(@NotNull DistributedPropertySet r) {
        for (Map.Entry<Class<? extends PropertySet>, PropertySet> entry : this.satellites.entrySet()) {
            r.addSatellite((Class)entry.getKey(), (PropertySet)entry.getValue());
        }
    }
    
    @Override
    public void copySatelliteInto(MessageContext r) {
        this.copySatelliteInto(r);
    }
    
    @Nullable
    @Override
    public <T extends PropertySet> T getSatellite(Class<T> satelliteClass) {
        T satellite = (T)this.satellites.get(satelliteClass);
        if (satellite != null) {
            return satellite;
        }
        for (PropertySet child : this.satellites.values()) {
            if (satelliteClass.isInstance(child)) {
                return (T)satelliteClass.cast(child);
            }
            if (DistributedPropertySet.class.isInstance(child)) {
                satellite = ((DistributedPropertySet)DistributedPropertySet.class.cast(child)).getSatellite(satelliteClass);
                if (satellite != null) {
                    return satellite;
                }
                continue;
            }
            else {
                continue;
            }
        }
        return null;
    }
    
    @Override
    public Map<Class<? extends PropertySet>, PropertySet> getSatellites() {
        return this.satellites;
    }
    
    @Override
    public Object get(Object key) {
        for (PropertySet child : this.satellites.values()) {
            if (child.supports(key)) {
                return child.get(key);
            }
            continue;
        }
        return super.get(key);
    }
    
    @Override
    public Object put(String key, Object value) {
        for (PropertySet child : this.satellites.values()) {
            if (child.supports(key)) {
                return child.put(key, value);
            }
            continue;
        }
        return super.put(key, value);
    }
    
    @Override
    public boolean containsKey(Object key) {
        if (this.viewthis.containsKey(key)) {
            return true;
        }
        for (PropertySet child : this.satellites.values()) {
            if (child.containsKey(key)) {
                return true;
            }
            continue;
        }
        return false;
    }
    
    @Override
    public boolean supports(Object key) {
        for (PropertySet child : this.satellites.values()) {
            if (child.supports(key)) {
                return true;
            }
            continue;
        }
        return super.supports(key);
    }
    
    @Override
    public Object remove(Object key) {
        for (PropertySet child : this.satellites.values()) {
            if (child.supports(key)) {
                return child.remove(key);
            }
            continue;
        }
        return super.remove(key);
    }
    
    @Override
    protected void createEntrySet(Set<Map.Entry<String, Object>> core) {
        super.createEntrySet(core);
        for (PropertySet child : this.satellites.values()) {
            ((BasePropertySet)child).createEntrySet(core);
        }
    }
    
    protected Map<String, Object> asMapLocal() {
        return this.viewthis;
    }
    
    protected boolean supportsLocal(Object key) {
        return super.supports(key);
    }
    
    @Override
    protected Map<String, Object> createView() {
        return new BaseDistributedPropertySet.DistributedMapView();
    }
    
    class DistributedMapView extends AbstractMap<String, Object>
    {
        @Override
        public Object get(Object key) {
            for (PropertySet child : BaseDistributedPropertySet.this.satellites.values()) {
                if (child.supports(key)) {
                    return child.get(key);
                }
                continue;
            }
            return BaseDistributedPropertySet.this.viewthis.get(key);
        }
        
        @Override
        public int size() {
            int size = BaseDistributedPropertySet.this.viewthis.size();
            for (PropertySet child : BaseDistributedPropertySet.this.satellites.values()) {
                size += child.asMap().size();
            }
            return size;
        }
        
        @Override
        public boolean containsKey(Object key) {
            if (BaseDistributedPropertySet.this.viewthis.containsKey(key)) {
                return true;
            }
            for (PropertySet child : BaseDistributedPropertySet.this.satellites.values()) {
                if (child.containsKey(key)) {
                    return true;
                }
                continue;
            }
            return false;
        }
        
        @Override
        public Set<Map.Entry<String, Object>> entrySet() {
            Set<Map.Entry<String, Object>> entries = new HashSet();
            for (PropertySet child : BaseDistributedPropertySet.this.satellites.values()) {
                for (Map.Entry<String, Object> entry : child.asMap().entrySet()) {
                    entries.add(new AbstractMap.SimpleImmutableEntry(entry.getKey(), entry.getValue()));
                }
            }
            for (Map.Entry<String, Object> entry2 : BaseDistributedPropertySet.this.viewthis.entrySet()) {
                entries.add(new AbstractMap.SimpleImmutableEntry(entry2.getKey(), entry2.getValue()));
            }
            return entries;
        }
        
        @Override
        public Object put(String key, Object value) {
            for (PropertySet child : BaseDistributedPropertySet.this.satellites.values()) {
                if (child.supports(key)) {
                    return child.put(key, value);
                }
                continue;
            }
            return BaseDistributedPropertySet.this.viewthis.put(key, value);
        }
        
        @Override
        public void clear() {
            BaseDistributedPropertySet.this.satellites.clear();
            BaseDistributedPropertySet.this.viewthis.clear();
        }
        
        @Override
        public Object remove(Object key) {
            for (PropertySet child : BaseDistributedPropertySet.this.satellites.values()) {
                if (child.supports(key)) {
                    return child.remove(key);
                }
                continue;
            }
            return BaseDistributedPropertySet.this.viewthis.remove(key);
        }
    }
}
